"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.playwrightFetch = playwrightFetch;

var _httpsProxyAgent = require("https-proxy-agent");

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var url = _interopRequireWildcard(require("url"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
async function playwrightFetch(context, params) {
  try {
    var _params$headers;

    const cookies = await context.cookies(params.url);
    const valueArray = cookies.map(c => `${c.name}=${c.value}`);
    const clientCookie = (_params$headers = params.headers) === null || _params$headers === void 0 ? void 0 : _params$headers['cookie'];
    if (clientCookie) valueArray.unshift(clientCookie);
    const cookieHeader = valueArray.join('; ');

    if (cookieHeader) {
      if (!params.headers) params.headers = {};
      params.headers['cookie'] = cookieHeader;
    }

    if (!params.method) params.method = 'GET';
    let agent;

    if (context._options.proxy) {
      // TODO: support bypass proxy
      const proxyOpts = url.parse(context._options.proxy.server);
      if (context._options.proxy.username) proxyOpts.auth = `${context._options.proxy.username}:${context._options.proxy.password || ''}`;
      agent = new _httpsProxyAgent.HttpsProxyAgent(proxyOpts);
    } // TODO(https://github.com/microsoft/playwright/issues/8381): set user agent


    const response = await (0, _nodeFetch.default)(params.url, {
      method: params.method,
      headers: params.headers,
      body: params.postData,
      agent
    });
    const body = await response.buffer();
    const setCookies = response.headers.raw()['set-cookie'];

    if (setCookies) {
      const url = new URL(response.url); // https://datatracker.ietf.org/doc/html/rfc6265#section-5.1.4

      const defaultPath = '/' + url.pathname.split('/').slice(0, -1).join('/');
      const cookies = [];

      for (const header of setCookies) {
        // Decode cookie value?
        const cookie = parseCookie(header);
        if (!cookie) continue;
        if (!cookie.domain) cookie.domain = url.hostname;
        if (!canSetCookie(cookie.domain, url.hostname)) continue; // https://datatracker.ietf.org/doc/html/rfc6265#section-5.2.4

        if (!cookie.path || !cookie.path.startsWith('/')) cookie.path = defaultPath;
        cookies.push(cookie);
      }

      if (cookies.length) await context.addCookies(cookies);
    }

    const headers = [];

    for (const [name, value] of response.headers.entries()) headers.push({
      name,
      value
    });

    return {
      fetchResponse: {
        url: response.url,
        status: response.status,
        statusText: response.statusText,
        headers,
        body
      }
    };
  } catch (e) {
    return {
      error: String(e)
    };
  }
}

function canSetCookie(cookieDomain, hostname) {
  // TODO: check public suffix list?
  hostname = '.' + hostname;
  if (!cookieDomain.startsWith('.')) cookieDomain = '.' + cookieDomain;
  return hostname.endsWith(cookieDomain);
}

function parseCookie(header) {
  const pairs = header.split(';').filter(s => s.trim().length > 0).map(p => p.split('=').map(s => s.trim()));
  if (!pairs.length) return null;
  const [name, value] = pairs[0];
  const cookie = {
    name,
    value,
    domain: '',
    path: '',
    expires: -1,
    httpOnly: false,
    secure: false,
    sameSite: 'Lax' // None for non-chromium

  };

  for (let i = 1; i < pairs.length; i++) {
    const [name, value] = pairs[i];

    switch (name.toLowerCase()) {
      case 'expires':
        const expiresMs = +new Date(value);
        if (isFinite(expiresMs)) cookie.expires = expiresMs / 1000;
        break;

      case 'max-age':
        const maxAgeSec = parseInt(value, 10);
        if (isFinite(maxAgeSec)) cookie.expires = Date.now() / 1000 + maxAgeSec;
        break;

      case 'domain':
        cookie.domain = value || '';
        break;

      case 'path':
        cookie.path = value || '';
        break;

      case 'secure':
        cookie.secure = true;
        break;

      case 'httponly':
        cookie.httpOnly = true;
        break;
    }
  }

  return cookie;
}